@@ -26,33 +26,13 @@
 #include <pdal/Dimension.hpp>
 #include <pdal/Metadata.hpp>
 #include <pdal/StageFactory.hpp>
+#include <pdal/io/LasHeader.hpp>
+#include <pdal/io/LasReader.hpp>
 #include <pdal/util/Algorithm.hpp>
 #include <pdal/util/Bounds.hpp>
 #include <pdal/util/FileUtils.hpp>
 #include <pdal/util/ProgramArgs.hpp>
 
-namespace
-{
-
-pdal::Dimension::Type getDimensionType(const std::string& name)
-{
-    using namespace pdal;
-
-    if (name == untwine::UntwineBitsDimName)
-        return untwine::UntwineBitsType;
-
-    Dimension::Type type = Dimension::Type::Double;
-    try
-    {
-        type = Dimension::defaultType(Dimension::id(name));
-    }
-    catch (pdal_error&)
-    {}
-
-    return type;
-}
-
-} // unnamed namespace
 
 namespace untwine
 {
@@ -77,12 +57,26 @@ void Epf::run(ProgressWriter& progress)
 
     m_grid.setCubic(m_b.opts.doCube);
 
-    // Create the file infos. As each info is created, the N x N x N grid is expanded to
+    // Create the file infos.
+    std::vector<FileInfo> fileInfos;
+    createFileInfos(m_b.opts.inputFiles, fileInfos);
+
+    pdal::PointLayout outputLayout;
+
+    filterDims(fileInfos, m_b.opts.dimNames);
+    determineDims(fileInfos, outputLayout);
+    determineOffset(fileInfos);
+    m_b.srs = determineSrs(fileInfos);
+
+    // Setup grid and point count. For each file info the N x N x N grid is expanded to
     // hold all the points. If the number of points seems too large, N is expanded to N + 1.
     // The correct N is often wrong, especially for some areas where things are more dense.
-    std::vector<FileInfo> fileInfos;
-    point_count_t totalPoints = createFileInfo(m_b.opts.inputFiles, m_b.opts.dimNames, fileInfos);
-
+    PointCount totalPoints = 0;
+    for (const FileInfo& info : fileInfos)
+    {
+        m_grid.expand(info.bounds, info.numPoints);
+        totalPoints += info.numPoints;
+    }
     if (m_b.opts.level != -1)
         m_grid.resetLevel(m_b.opts.level);
 
@@ -90,27 +84,7 @@ void Epf::run(ProgressWriter& progress)
     if (fileInfos.size() > m_b.opts.fileLimit)
         fileInfos.resize(m_b.opts.fileLimit);
 
-    // Stick all the dimension names from each input file in a set.
-    std::unordered_set<std::string> allDimNames;
-    for (const FileInfo& fi : fileInfos)
-        for (const FileDimInfo& fdi : fi.dimInfo)
-            allDimNames.insert(fdi.name);
-
-    // Create an OUTPUT layout.
-    // Register the dimensions, either as the default type or double if we don't know
-    // what it is.
-    PointLayoutPtr layout(new PointLayout());
-    for (std::string dimName : allDimNames)
-    {
-        // If this is a "bit" dimension, don't add it, but instead register the proxy
-        // untwine bits dimension.
-        if (isUntwineBitsDim(dimName))
-            dimName = UntwineBitsDimName;
-        layout->registerOrAssignDim(dimName, getDimensionType(dimName));
-    }
-    layout->finalize();
-
-    // Fill in dim info now that the layout is finalized.
+    // Fill in dim info.
     for (FileInfo& fi : fileInfos)
     {
         for (FileDimInfo& di : fi.dimInfo)
@@ -120,20 +94,20 @@ void Epf::run(ProgressWriter& progress)
             if (bitPos != -1)
             {
                 di.type = UntwineBitsType;
-                di.offset = layout->dimOffset(layout->findDim(UntwineBitsDimName));
+                di.offset = outputLayout.dimOffset(outputLayout.findDim(UntwineBitsDimName));
                 di.shift = bitPos;
             }
             else
             {
-                Dimension::Id dim = layout->findDim(di.name);
-                di.type = layout->dimType(dim);
-                di.offset = layout->dimOffset(dim);
+                Dimension::Id dim = outputLayout.findDim(di.name);
+                di.type = outputLayout.dimType(dim);
+                di.offset = outputLayout.dimOffset(dim);
             }
         }
     }
 
     // Make a writer with NumWriters threads.
-    m_writer.reset(new Writer(m_b.opts.tempDir, NumWriters, layout->pointSize()));
+    m_writer.reset(new Writer(m_b.opts.tempDir, NumWriters, outputLayout.pointSize()));
 
     // Sort file infos so the largest files come first. This helps to make sure we don't delay
     // processing big files that take the longest (use threads more efficiently).
@@ -146,7 +120,7 @@ void Epf::run(ProgressWriter& progress)
     m_pool.trap(true, "Unknown error in FileProcessor");
     for (const FileInfo& fi : fileInfos)
     {
-        int pointSize = layout->pointSize();
+        int pointSize = outputLayout.pointSize();
         m_pool.add([&fi, &progress, pointSize, this]()
         {
             FileProcessor fp(fi, pointSize, m_grid, m_writer.get(), progress);
@@ -161,7 +135,7 @@ void Epf::run(ProgressWriter& progress)
     m_writer->stop();
 
     // If the FileProcessors had an error, throw.
-    std::vector<std::string> errors = m_pool.clearErrors();
+    StringList errors = m_pool.clearErrors();
     if (errors.size())
         throw FatalError(errors.front());
 
@@ -179,13 +153,13 @@ void Epf::run(ProgressWriter& progress)
 
     // Make a new writer since we stopped the old one. Could restart, but why bother with
     // extra code...
-    m_writer.reset(new Writer(m_b.opts.tempDir, 4, layout->pointSize()));
+    m_writer.reset(new Writer(m_b.opts.tempDir, 4, outputLayout.pointSize()));
     m_pool.trap(true, "Unknown error in Reprocessor");
     for (auto& t : totals)
     {
         VoxelKey key = t.first;
         int numPoints = t.second;
-        int pointSize = layout->pointSize();
+        int pointSize = outputLayout.pointSize();
         std::string tempDir = m_b.opts.tempDir;
 
         // Create a reprocessor thread. Note that the grid is copied by value and
@@ -205,40 +179,40 @@ void Epf::run(ProgressWriter& progress)
 
     m_writer->stop();
 
-    fillMetadata(layout);
+    fillMetadata(outputLayout);
 }
 
-void Epf::fillMetadata(const pdal::PointLayoutPtr layout)
+void Epf::fillMetadata(const pdal::PointLayout& layout)
 {
     using namespace pdal;
 
     // Info to be passed to sampler.
     m_b.bounds = m_grid.processingBounds();
     m_b.trueBounds = m_grid.conformingBounds();
-    if (m_srsFileInfo.valid())
-        m_b.srs = m_srsFileInfo.srs;
+
     m_b.pointSize = 0;
 
     // Set the pointFormatId based on whether or not colors exist in the file
-    if (layout->hasDim(Dimension::Id::Infrared))
+    if (layout.hasDim(Dimension::Id::Infrared))
         m_b.pointFormatId = 8;
-    else if (layout->hasDim(Dimension::Id::Red) ||
-             layout->hasDim(Dimension::Id::Green) ||
-             layout->hasDim(Dimension::Id::Blue))
+    else if (layout.hasDim(Dimension::Id::Red) ||
+             layout.hasDim(Dimension::Id::Green) ||
+             layout.hasDim(Dimension::Id::Blue))
         m_b.pointFormatId = 7;
     else
         m_b.pointFormatId = 6;
 
-    for (Dimension::Id id : layout->dims())
+    for (Dimension::Id id : layout.dims())
     {
         FileDimInfo di;
-        di.name = layout->dimName(id);
-        di.type = layout->dimType(id);
-        di.offset = layout->dimOffset(id);
+        di.name = layout.dimName(id);
+        di.type = layout.dimType(id);
+        di.offset = layout.dimOffset(id);
         di.extraDim = isExtraDim(di.name);
         m_b.pointSize += pdal::Dimension::size(di.type);
         m_b.dimInfo.push_back(di);
     }
+
     auto calcScale = [](double scale, double low, double high)
     {
         if (scale > 0)
@@ -271,9 +245,7 @@ void Epf::fillMetadata(const pdal::PointLayoutPtr layout)
 
     // Preserve offsets if we have them and --single_file with single input is used
     if (!m_b.preserveHeaderFields() ||
-            std::isnan(m_b.offset[0]) ||
-            std::isnan(m_b.offset[1]) ||
-            std::isnan(m_b.offset[2]))
+        std::isnan(m_b.offset[0]) || std::isnan(m_b.offset[1]) || std::isnan(m_b.offset[2]))
     {
         m_b.offset[0] = calcOffset(m_b.trueBounds.minx, m_b.trueBounds.maxx, m_b.scale[0]);
         m_b.offset[1] = calcOffset(m_b.trueBounds.miny, m_b.trueBounds.maxy, m_b.scale[1]);
@@ -281,25 +253,11 @@ void Epf::fillMetadata(const pdal::PointLayoutPtr layout)
     }
 }
 
-PointCount Epf::createFileInfo(const StringList& input, StringList dimNames,
-    std::vector<FileInfo>& fileInfos)
+void Epf::createFileInfos(const StringList& input, std::vector<FileInfo>& fileInfos)
 {
     using namespace pdal;
 
-    std::vector<FileInfo> tempFileInfos;
-    std::vector<std::string> filenames;
-    PointCount totalPoints = 0;
-
-    // If there are some dim names specified, make sure they contain X, Y and Z and that
-    // they're all uppercase.
-    if (!dimNames.empty())
-    {
-        for (std::string& d : dimNames)
-            d = Utils::toupper(d);
-        for (const std::string xyz : { "X", "Y", "Z" })
-            if (!Utils::contains(dimNames, xyz))
-                dimNames.push_back(xyz);
-    }
+    StringList filenames;
 
     // If any of the specified input files is a directory, get the names of the files
     // in the directory and add them.
@@ -307,17 +265,13 @@ PointCount Epf::createFileInfo(const StringList& input, StringList dimNames,
     {
         if (FileUtils::isDirectory(filename))
         {
-            std::vector<std::string> dirfiles = directoryList(filename);
+            StringList dirfiles = directoryList(filename);
             filenames.insert(filenames.end(), dirfiles.begin(), dirfiles.end());
         }
         else
             filenames.push_back(filename);
     }
 
-    std::vector<double> xOffsets;
-    std::vector<double> yOffsets;
-    std::vector<double> zOffsets;
-
     // Determine a driver for each file and get a preview of the file.  If we couldn't
     // Create a FileInfo object containing the file bounds, dimensions, filename and
     // associated driver.  Expand our grid by the bounds and file point count.
@@ -328,150 +282,243 @@ PointCount Epf::createFileInfo(const StringList& input, StringList dimNames,
         if (driver.empty())
             throw FatalError("Can't infer reader for '" + filename + "'.");
         Stage *s = factory.createStage(driver);
+
         pdal::Options opts;
         opts.add("filename", filename);
+        if (driver == "readers.las")
+            opts.add("nosrs", m_b.opts.no_srs);
         s->setOptions(opts);
 
-        QuickInfo qi = s->preview();
-
-        // Detect LAS input with LAS version < 4 so that we can handle the legacy
-        // classification bits.
-        if (!qi.valid())
-            throw FatalError("Couldn't get quick info for '" + filename + "'.");
-
-        // Get scale values from the reader if they exist.
-        pdal::MetadataNode root = s->getMetadata();
-        pdal::MetadataNode m = root.findChild("scale_x");
-        if (m.valid())
-            m_b.scale[0] = (std::max)(m_b.scale[0], m.value<double>());
-        m = root.findChild("scale_y");
-        if (m.valid())
-            m_b.scale[1] = (std::max)(m_b.scale[1], m.value<double>());
-        m = root.findChild("scale_z");
-        if (m.valid())
-            m_b.scale[2] = (std::max)(m_b.scale[2], m.value<double>());
-        m = root.findChild("offset_x");
-        if (m.valid())
-            xOffsets.push_back(m.value<double>());
-        m = root.findChild("offset_y");
-        if (m.valid())
-            yOffsets.push_back(m.value<double>());
-        m = root.findChild("offset_z");
-        if (m.valid())
-            zOffsets.push_back(m.value<double>());
-
-        if (m_b.preserveHeaderFields())
+        FileInfo fi;
+        fi.filename = filename;
+        fi.driver = driver;
+        fi.no_srs = m_b.opts.no_srs;
+        if (driver == "readers.las")
         {
-            m = root.findChild("global_encoding");
-            if (m.valid())
-                m_b.globalEncoding = m.value<int>();
-            m = root.findChild("creation_doy");
-            if (m.valid())
-                m_b.creationDoy = m.value<int>();
-            m = root.findChild("creation_year");
-            if (m.valid())
-                m_b.creationYear = m.value<int>();
-            m = root.findChild("filesource_id");
-            if (m.valid())
-                m_b.fileSourceId = m.value<int>();
-            m = root.findChild("software_id");
-            if (m.valid())
-                m_b.generatingSoftware = m.value<std::string>();
-            m = root.findChild("system_id");
-            if (m.valid())
-                m_b.systemId = m.value<std::string>();
+            const std::vector<FileInfo>& infos = processLas(*dynamic_cast<LasReader *>(s), fi);
+            fileInfos.insert(fileInfos.begin(), infos.begin(), infos.end());
         }
         else
+            fileInfos.push_back(processGeneric(*s, fi));
+    }
+}
+
+void Epf::filterDims(std::vector<FileInfo>& infos, StringList allowedDims)
+{
+    if (allowedDims.empty())
+        return;
+
+    // If there are some dim names specified, make sure they contain X, Y and Z and that
+    // they're all uppercase.
+    for (std::string& d : allowedDims)
+        d = pdal::Utils::toupper(d);
+    for (const std::string xyz : { "X", "Y", "Z" })
+        if (!pdal::Utils::contains(allowedDims, xyz))
+            allowedDims.push_back(xyz);
+
+    // Remove dimensions not in the allowed list.
+    for (FileInfo& info : infos)
+        for (auto it = info.dimInfo.begin(); it != info.dimInfo.end(); ++it)
         {
-            std::time_t now;
-            std::time(&now);
-            std::tm* ptm = std::gmtime(&now);
-            if (ptm)
+            FileDimInfo& fdi = *it;
+            if (!pdal::Utils::contains(allowedDims, fdi.name))
+                it = info.dimInfo.erase(it);
+        }
+}
+
+void Epf::determineDims(std::vector<FileInfo>& infos, pdal::PointLayout& layout)
+{
+    using namespace pdal;
+
+    StringList untypedDimNames;
+
+    // Register dimensions in the OUTPUT layout, transforming dimensions that are packed
+    // into the "untwine bits" dimension.
+    for (FileInfo& info : infos)
+        for (FileDimInfo dim : info.dimInfo)
+        {
+            std::string name = dim.name;
+            Dimension::Type type = dim.type;
+
+            if (type == Dimension::Type::None)
+                untypedDimNames.push_back(name);
+            else
             {
-                m_b.creationDoy = ptm->tm_yday + 1;
-                m_b.creationYear = ptm->tm_year + 1900;
+                if (isUntwineBitsDim(name))
+                {
+                    name = UntwineBitsDimName;
+                    type = UntwineBitsType;
+                }
+                layout.registerOrAssignDim(name, type);
             }
         }
 
-        FileInfo fi;
-        fi.bounds = qi.m_bounds;
-        fi.numPoints = qi.m_pointCount;
-        fi.filename = filename;
-        fi.driver = driver;
-
-        // Detect LAS input with LAS version < 4 so that we can handle the legacy
-        // classification bits.
-        if (driver == "readers.las")
+    // Register dimensions that didn't come from LAS files if their names don't match
+    // those that are already in the output layout.
+    for (std::string name : untypedDimNames)
+    {
+        if (layout.findDim(name) == Dimension::Id::Unknown)
         {
-            pdal::MetadataNode minor = root.findChild("minor_version");
-            pdal::MetadataNode major = root.findChild("major_version");
-            if (minor.valid() && major.valid())
-                fi.fileVersion = 10 * major.value<int>() + minor.value<int>();
+            Dimension::Type type = Dimension::Type::Double;
+            if (isUntwineBitsDim(name))
+            {
+                name = UntwineBitsDimName;
+                type = UntwineBitsType;
+            }
+            else
+            {
+                try
+                {
+                    type = Dimension::defaultType(Dimension::id(name));
+                }
+                catch (pdal_error&)
+                {}
+            }
+            layout.registerOrAssignDim(name, type);
         }
-
-        // Accept dimension names if there are no limits or this name is in the list
-        // of desired dimensions.
-        for (const std::string& name : qi.m_dimNames)
-            if (dimNames.empty() || Utils::contains(dimNames, Utils::toupper(name)))
-                fi.dimInfo.push_back(FileDimInfo(name));
-
-        if (m_srsFileInfo.valid() && m_srsFileInfo.srs != qi.m_srs)
-            std::cerr << "Files have mismatched SRS values. Using SRS from '" <<
-                m_srsFileInfo.filename << "'.\n";
-        fi.srs = qi.m_srs;
-        tempFileInfos.push_back(fi);
-        if (!m_srsFileInfo.valid() && qi.m_srs.valid())
-            m_srsFileInfo = fi;
-
-        m_grid.expand(qi.m_bounds, qi.m_pointCount);
-        totalPoints += fi.numPoints;
     }
 
-    // If we had an offset from the input, choose one in the middle of the list of offsets.
+    layout.finalize();
+}
+
+// Determine a reasonable offset for the output data.
+void Epf::determineOffset(const std::vector<FileInfo>& infos)
+{
+    std::vector<double> xOffsets;
+    std::vector<double> yOffsets;
+    std::vector<double> zOffsets;
+
+    for (const auto& fi : infos)
+    {
+        xOffsets.push_back(fi.offsets[0]);
+        yOffsets.push_back(fi.offsets[1]);
+        zOffsets.push_back(fi.offsets[2]);
+    }
+    std::sort(xOffsets.begin(), xOffsets.end());
+    std::sort(yOffsets.begin(), yOffsets.end());
+    std::sort(zOffsets.begin(), zOffsets.end());
     if (xOffsets.size())
-    {
-        std::sort(xOffsets.begin(), xOffsets.end());
         m_b.offset[0] = xOffsets[xOffsets.size() / 2];
-    }
     if (yOffsets.size())
-    {
-        std::sort(yOffsets.begin(), yOffsets.end());
         m_b.offset[1] = yOffsets[yOffsets.size() / 2];
-    }
     if (zOffsets.size())
-    {
-        std::sort(zOffsets.begin(), zOffsets.end());
         m_b.offset[2] = zOffsets[zOffsets.size() / 2];
+}
+
+pdal::SpatialReference Epf::determineSrs(const std::vector<FileInfo>& infos)
+{
+    // Here FileInfo is just convenient as it has both SRS and filename.
+    FileInfo srsFileInfo;
+
+    for (const auto& fi : infos)
+    {
+        if (fi.srs.valid())
+        {
+            if (!srsFileInfo.srs.valid())
+                srsFileInfo = fi;
+            else if (srsFileInfo.srs != fi.srs)
+                std::cerr << "Files have mismatched SRS values. Using SRS from '" <<
+                    srsFileInfo.filename << "'.\n";
+        }
+    }
+    return srsFileInfo.srs;
+}
+
+std::vector<FileInfo> Epf::processLas(pdal::LasReader& r, FileInfo fi)
+{
+    using namespace pdal;
+
+    PointTable t;
+    r.prepare(t);
+
+    const LasHeader& h = r.header();
+
+    fi.bounds = h.getBounds();
+    fi.numPoints = h.pointCount();
+
+    m_b.scale[0] = (std::max)(m_b.scale[0], h.scaleX());
+    m_b.scale[1] = (std::max)(m_b.scale[0], h.scaleY());
+    m_b.scale[2] = (std::max)(m_b.scale[0], h.scaleZ());
+
+    fi.offsets[0] = h.offsetX();
+    fi.offsets[1] = h.offsetY();
+    fi.offsets[2] = h.offsetZ();
+
+    if (m_b.preserveHeaderFields())
+    {
+        m_b.globalEncoding = h.globalEncoding();
+        m_b.creationDoy = h.creationDOY();
+        m_b.creationYear = h.creationYear();
+        m_b.generatingSoftware = h.softwareId();
+        m_b.systemId = h.systemId();
+        m_b.fileSourceId = h.fileSourceId();
+    }
+    else
+        calcCreationDay();
+
+    // Detect LAS input with LAS version < 4 so that we can handle the legacy
+    // classification bits.
+    fi.fileVersion = 10 * h.versionMajor() + h.versionMinor();
+
+    PointLayoutPtr layout = t.layout();
+    for (pdal::Dimension::Id id : layout->dims())
+    {
+        const std::string& name = layout->dimName(id);
+        Dimension::Type type = layout->dimType(id);
+        fi.dimInfo.push_back(FileDimInfo(name, type));
     }
 
-    // If we have LAS start capability, break apart file infos into chunks of size 5 million.
+    std::vector<FileInfo> fileInfos;
+    // If we have LAS start capability, break apart file info into chunks of size 5 million.
 #ifdef PDAL_LAS_START
     PointCount ChunkSize = 5'000'000;
-    for (const FileInfo& fi : tempFileInfos)
-    {
-        if (fi.driver != "readers.las" || fi.numPoints < ChunkSize)
-        {
-            fileInfos.push_back(fi);
-            continue;
-        }
-        PointCount remaining = fi.numPoints;
-        pdal::PointId start = 0;
-        while (remaining)
-        {
-            FileInfo lasFi(fi);
-            lasFi.numPoints = (std::min)(ChunkSize, remaining);
-            lasFi.start = start;
-            fileInfos.push_back(lasFi);
 
-            start += ChunkSize;
-            remaining -= lasFi.numPoints;
-        }
+    PointCount remaining = fi.numPoints;
+    PointId start = 0;
+
+    while (remaining)
+    {
+        FileInfo lasFi(fi);
+        lasFi.numPoints = (std::min)(ChunkSize, remaining);
+        lasFi.start = start;
+        fileInfos.push_back(lasFi);
+
+        start += ChunkSize;
+        remaining -= lasFi.numPoints;
     }
 #else
-    fileInfos = std::move(tempFileInfos);
+    fileInfos.push_back(fi);
 #endif
+    return fileInfos;
+}
 
-    return totalPoints;
+FileInfo Epf::processGeneric(pdal::Stage& s, FileInfo fi)
+{
+    pdal::QuickInfo qi = s.preview();
+
+    if (!qi.valid())
+        throw FatalError("Couldn't get quick info for '" + fi.filename + "'.");
+
+    fi.bounds = qi.m_bounds;
+    fi.numPoints = qi.m_pointCount;
+
+    for (const std::string& name : qi.m_dimNames)
+        fi.dimInfo.push_back(FileDimInfo(name));
+
+    calcCreationDay();
+    return fi;
+}
+
+void Epf::calcCreationDay()
+{
+    std::time_t now;
+    std::time(&now);
+    std::tm* ptm = std::gmtime(&now);
+    if (ptm)
+    {
+        m_b.creationDoy = ptm->tm_yday + 1;
+        m_b.creationYear = ptm->tm_year + 1900;
+    }
 }
 
 } // namespace epf
