@@ -199,7 +199,8 @@ QVariantMap QgsJoinByAttributeAlgorithm::processAlgorithm( const QVariantMap &pa
 
     // only keep selected attributes
     QgsAttributes attributes;
-    for ( int j = 0; j < feat.attributes().count(); ++j )
+    const int attributeCount = feat.attributeCount();
+    for ( int j = 0; j < attributeCount; ++j )
     {
       if ( ! fields2Indices.contains( j ) )
         continue;
