@@ -364,6 +364,8 @@ QgsIdentifyResultsDialog::QgsIdentifyResultsDialog( QgsMapCanvas *canvas, QWidge
 
   mOpenFormAction->setDisabled( true );
 
+  lstResults->setVerticalScrollMode( QListView::ScrollMode::ScrollPerPixel );
+
   QgsSettings mySettings;
   mDock = new QgsDockWidget( tr( "Identify Results" ), QgisApp::instance() );
   mDock->setObjectName( QStringLiteral( "IdentifyResultsDock" ) );
@@ -434,9 +436,6 @@ QgsIdentifyResultsDialog::QgsIdentifyResultsDialog( QgsMapCanvas *canvas, QWidge
   mPlot->setSizePolicy( sizePolicy );
   mPlot->updateGeometry();
 
-  connect( lstResults, &QTreeWidget::itemExpanded,
-           this, &QgsIdentifyResultsDialog::itemExpanded );
-
   connect( lstResults, &QTreeWidget::currentItemChanged,
            this, &QgsIdentifyResultsDialog::handleCurrentItemChanged );
 
@@ -743,7 +742,7 @@ QgsIdentifyResultsFeatureItem *QgsIdentifyResultsDialog::createFeatureItem( QgsV
       continue;
     }
     QString defVal;
-    if ( fields.fieldOrigin( i ) == QgsFields::OriginProvider && vlayer->dataProvider() )
+    if ( fields.fieldOrigin( i ) == Qgis::FieldOrigin::Provider && vlayer->dataProvider() )
       defVal = vlayer->dataProvider()->defaultValueClause( fields.fieldOriginIndex( i ) );
 
     const QString originalValue = defVal == attrs.at( i ) ? defVal : fields.at( i ).displayString( attrs.at( i ) );
@@ -969,7 +968,7 @@ void QgsIdentifyResultsDialog::addFeature( QgsRasterLayer *layer,
 
     // Add all supported formats, best first. HTML is considered the best because
     // it usually holds most information.
-    const int capabilities = layer->dataProvider()->capabilities();
+    const Qgis::RasterInterfaceCapabilities capabilities = layer->dataProvider()->capabilities();
     static const QList<Qgis::RasterIdentifyFormat> formats
     {
       Qgis::RasterIdentifyFormat::Html,
@@ -978,7 +977,7 @@ void QgsIdentifyResultsDialog::addFeature( QgsRasterLayer *layer,
       Qgis::RasterIdentifyFormat::Value };
     for ( const auto &f : formats )
     {
-      if ( !( QgsRasterDataProvider::identifyFormatToCapability( f ) & capabilities ) )
+      if ( !( capabilities & QgsRasterDataProvider::identifyFormatToCapability( f ) ) )
         continue;
       formatCombo->addItem( QgsRasterDataProvider::identifyFormatLabel( f ), QVariant::fromValue( f ) );
       formatCombo->setItemData( formatCombo->count() - 1, QVariant::fromValue( qobject_cast<QObject *>( layer ) ), Qt::UserRole + 1 );
@@ -1013,7 +1012,7 @@ void QgsIdentifyResultsDialog::addFeature( QgsRasterLayer *layer,
   // add feature attributes
   if ( feature.isValid() )
   {
-    QgsDebugMsgLevel( QStringLiteral( "fields size = %1 attributes size = %2" ).arg( fields.size() ).arg( feature.attributes().size() ), 2 );
+    QgsDebugMsgLevel( QStringLiteral( "fields size = %1 attributes size = %2" ).arg( fields.size() ).arg( feature.attributeCount() ), 2 );
     const QgsAttributes attrs = feature.attributes();
     for ( int i = 0; i < attrs.count(); ++i )
     {
@@ -1026,7 +1025,7 @@ void QgsIdentifyResultsDialog::addFeature( QgsRasterLayer *layer,
       bool isString = false;
       if ( value.isValid( ) )
       {
-        if ( value.type() == QVariant::Double )
+        if ( value.userType() == QMetaType::Type::Double )
         {
           bool ok;
           const double val( value.toDouble( &ok ) );
@@ -1048,7 +1047,7 @@ void QgsIdentifyResultsDialog::addFeature( QgsRasterLayer *layer,
             formattedValue = QLocale().toString( val, 'f', precision );
           }
         }
-        else if ( value.type() == QVariant::Int )
+        else if ( value.userType() == QMetaType::Type::Int )
         {
           bool ok;
           const double val( value.toInt( &ok ) );
@@ -1057,7 +1056,7 @@ void QgsIdentifyResultsDialog::addFeature( QgsRasterLayer *layer,
             formattedValue =  QLocale().toString( val, 'f', 0 );
           }
         }
-        else if ( value.type() == QVariant::LongLong )
+        else if ( value.userType() == QMetaType::Type::LongLong )
         {
           bool ok;
           const double val( value.toLongLong( &ok ) );
@@ -1437,7 +1436,6 @@ void QgsIdentifyResultsDialog::addFeature( QgsTiledSceneLayer *layer,
   }
 }
 
-
 void QgsIdentifyResultsDialog::editingToggled()
 {
   QTreeWidgetItem *layItem = layerItem( sender() );
@@ -1982,33 +1980,37 @@ QgsTiledSceneLayer *QgsIdentifyResultsDialog::tiledSceneLayer( QTreeWidgetItem *
   return qobject_cast<QgsTiledSceneLayer *>( item->data( 0, Qt::UserRole ).value<QObject *>() );
 }
 
-QTreeWidgetItem *QgsIdentifyResultsDialog::retrieveAttributes( QTreeWidgetItem *item, QgsAttributeMap &attributes, int &idx )
+QgsAttributeMap QgsIdentifyResultsDialog::retrieveAttributes( QTreeWidgetItem *item )
 {
   QTreeWidgetItem *featItem = featureItem( item );
   if ( !featItem )
-    return nullptr;
+    return {};
 
-  idx = -1;
-
-  attributes.clear();
+  QgsAttributeMap attributes;
   for ( int i = 0; i < featItem->childCount(); i++ )
   {
     QTreeWidgetItem *item = featItem->child( i );
     if ( item->childCount() > 0 )
       continue;
-    if ( item == lstResults->currentItem() )
-      idx = item->data( 0, Qt::UserRole + 1 ).toInt();
-    attributes.insert( item->data( 0, Qt::UserRole + 1 ).toInt(), item->data( 1, REPRESENTED_VALUE_ROLE ) );
+
+    attributes.insert( item->data( 0, Qt::UserRole + 1 ).toInt(),
+                       retrieveAttribute( item ) );
   }
 
-  return featItem;
+  return attributes;
 }
 
-void QgsIdentifyResultsDialog::itemExpanded( QTreeWidgetItem *item )
+QVariant QgsIdentifyResultsDialog::retrieveAttribute( QTreeWidgetItem *item )
 {
-  Q_UNUSED( item )
-  // column width is now stored in settings
-  //expandColumnsToFit();
+  if ( !item )
+    return QVariant();
+
+  // prefer represented values, if available.
+  const QVariant representedValue = item->data( 1, REPRESENTED_VALUE_ROLE );
+  if ( !QgsVariantUtils::isNull( representedValue ) )
+    return representedValue;
+
+  return item->data( 1, Qt::DisplayRole );
 }
 
 void QgsIdentifyResultsDialog::handleCurrentItemChanged( QTreeWidgetItem *current, QTreeWidgetItem *previous )
@@ -2378,7 +2380,8 @@ void QgsIdentifyResultsDialog::collapseAll()
 void QgsIdentifyResultsDialog::copyAttributeValue()
 {
   QClipboard *clipboard = QApplication::clipboard();
-  const QString text = lstResults->currentItem()->data( 1, REPRESENTED_VALUE_ROLE ).toString();
+  const QVariant attributeValue = retrieveAttribute( lstResults->currentItem() );
+  const QString text = attributeValue.toString();
   QgsDebugMsgLevel( QStringLiteral( "set clipboard: %1" ).arg( text ), 2 );
   clipboard->setText( text );
 }
@@ -2397,12 +2400,8 @@ void QgsIdentifyResultsDialog::copyFeatureAttributes()
 
   if ( vlayer )
   {
-    int idx;
-    QgsAttributeMap attributes;
-    retrieveAttributes( lstResults->currentItem(), attributes, idx );
-
-    const QgsFields &fields = vlayer->fields();
-
+    const QgsAttributeMap attributes = retrieveAttributes( lstResults->currentItem() );
+    const QgsFields fields = vlayer->fields();
     for ( QgsAttributeMap::const_iterator it = attributes.constBegin(); it != attributes.constEnd(); ++it )
     {
       const int attrIdx = it.key();
