@@ -319,12 +319,13 @@ void QgsLayoutItemLegend::setCustomLayerTree( QgsLayerTree *rootGroup )
   mCustomLayerTree.reset( rootGroup );
 }
 
-void QgsLayoutItemLegend::ensureModelIsInitialized()
+void QgsLayoutItemLegend::ensureModelIsInitialized() const
 {
   if ( mDeferLegendModelInitialization )
   {
-    mDeferLegendModelInitialization = false;
-    setCustomLayerTree( mCustomLayerTree.release() );
+    QgsLayoutItemLegend *mutableThis = const_cast< QgsLayoutItemLegend * >( this );
+    mutableThis->mDeferLegendModelInitialization = false;
+    mutableThis->setCustomLayerTree( mutableThis->mCustomLayerTree.release() );
   }
 }
 
@@ -334,6 +335,12 @@ QgsLegendModel *QgsLayoutItemLegend::model()
   return mLegendModel.get();
 }
 
+const QgsLegendModel *QgsLayoutItemLegend::model() const
+{
+  ensureModelIsInitialized();
+  return mLegendModel.get();
+}
+
 void QgsLayoutItemLegend::setAutoUpdateModel( bool autoUpdate )
 {
   if ( autoUpdate == autoUpdateModel() )
@@ -1345,7 +1352,7 @@ bool QgsLayoutItemLegend::accept( QgsStyleEntityVisitorInterface *visitor ) cons
     }
     return true;
   };
-  return visit( mLegendModel->rootGroup( ) );
+  return visit( model()->rootGroup( ) );
 }
 
 bool QgsLayoutItemLegend::isRefreshing() const
