@@ -539,20 +539,20 @@ void QgsOfflineEditing::convertToOfflineLayer( QgsVectorLayer *layer, sqlite3 *d
       for ( const auto &field : providerFields )
       {
         QString dataType;
-        const QVariant::Type type = field.type();
-        if ( type == QVariant::Int || type == QVariant::LongLong )
+        const QMetaType::Type type = field.type();
+        if ( type == QMetaType::Type::Int || type == QMetaType::Type::LongLong )
         {
           dataType = QStringLiteral( "INTEGER" );
         }
-        else if ( type == QVariant::Double )
+        else if ( type == QMetaType::Type::Double )
         {
           dataType = QStringLiteral( "REAL" );
         }
-        else if ( type == QVariant::String )
+        else if ( type == QMetaType::Type::QString )
         {
           dataType = QStringLiteral( "TEXT" );
         }
-        else if ( type == QVariant::StringList  || type == QVariant::List )
+        else if ( type == QMetaType::Type::QStringList  || type == QMetaType::Type::QVariantList )
         {
           dataType = QStringLiteral( "TEXT" );
           showWarning( tr( "Field '%1' from layer %2 has been converted from a list to a string of comma-separated values." ).arg( field.name(), layer->name() ) );
@@ -705,27 +705,27 @@ void QgsOfflineEditing::convertToOfflineLayer( QgsVectorLayer *layer, sqlite3 *d
       for ( const auto &field : providerFields )
       {
         const QString fieldName( field.name() );
-        const QVariant::Type type = field.type();
+        const QMetaType::Type type = field.type();
         OGRFieldType ogrType( OFTString );
         OGRFieldSubType ogrSubType = OFSTNone;
-        if ( type == QVariant::Int )
+        if ( type == QMetaType::Type::Int )
           ogrType = OFTInteger;
-        else if ( type == QVariant::LongLong )
+        else if ( type == QMetaType::Type::LongLong )
           ogrType = OFTInteger64;
-        else if ( type == QVariant::Double )
+        else if ( type == QMetaType::Type::Double )
           ogrType = OFTReal;
-        else if ( type == QVariant::Time )
+        else if ( type == QMetaType::Type::QTime )
           ogrType = OFTTime;
-        else if ( type == QVariant::Date )
+        else if ( type == QMetaType::Type::QDate )
           ogrType = OFTDate;
-        else if ( type == QVariant::DateTime )
+        else if ( type == QMetaType::Type::QDateTime )
           ogrType = OFTDateTime;
-        else if ( type == QVariant::Bool )
+        else if ( type == QMetaType::Type::Bool )
         {
           ogrType = OFTInteger;
           ogrSubType = OFSTBoolean;
         }
-        else if ( type == QVariant::StringList || type == QVariant::List )
+        else if ( type == QMetaType::Type::QStringList || type == QMetaType::Type::QVariantList )
         {
           ogrType = OFTString;
           ogrSubType = OFSTJSON;
@@ -916,7 +916,7 @@ void QgsOfflineEditing::applyAttributesAdded( QgsVectorLayer *remoteLayer, sqlit
   const QList<QgsVectorDataProvider::NativeType> nativeTypes = provider->nativeTypes();
 
   // NOTE: uses last matching QVariant::Type of nativeTypes
-  QMap < QVariant::Type, QString /*typeName*/ > typeNameLookup;
+  QMap < QMetaType::Type, QString /*typeName*/ > typeNameLookup;
   for ( int i = 0; i < nativeTypes.size(); i++ )
   {
     const QgsVectorDataProvider::NativeType nativeType = nativeTypes.at( i );
@@ -987,20 +987,20 @@ void QgsOfflineEditing::applyFeaturesAdded( QgsVectorLayer *offlineLayer, QgsVec
       if ( remoteAttributeIndex == -1 )
         continue;
       QVariant attr = attrs.at( it );
-      if ( remoteLayer->fields().at( remoteAttributeIndex ).type() == QVariant::StringList )
+      if ( remoteLayer->fields().at( remoteAttributeIndex ).type() == QMetaType::Type::QStringList )
       {
-        if ( attr.type() == QVariant::StringList || attr.type() == QVariant::List )
+        if ( attr.userType() == QMetaType::Type::QStringList || attr.userType() == QMetaType::Type::QVariantList )
         {
           attr = attr.toStringList();
         }
         else
         {
-          attr = QgsJsonUtils::parseArray( attr.toString(), QVariant::String );
+          attr = QgsJsonUtils::parseArray( attr.toString(), QMetaType::Type::QString );
         }
       }
-      else if ( remoteLayer->fields().at( remoteAttributeIndex ).type() == QVariant::List )
+      else if ( remoteLayer->fields().at( remoteAttributeIndex ).type() == QMetaType::Type::QVariantList )
       {
-        if ( attr.type() == QVariant::StringList || attr.type() == QVariant::List )
+        if ( attr.userType() == QMetaType::Type::QStringList || attr.userType() == QMetaType::Type::QVariantList )
         {
           attr = attr.toList();
         }
@@ -1058,11 +1058,11 @@ void QgsOfflineEditing::applyAttributeValueChanges( QgsVectorLayer *offlineLayer
 
     const int remoteAttributeIndex = attrLookup[ values.at( i ).attr ];
     QVariant attr = values.at( i ).value;
-    if ( remoteLayer->fields().at( remoteAttributeIndex ).type() == QVariant::StringList )
+    if ( remoteLayer->fields().at( remoteAttributeIndex ).type() == QMetaType::Type::QStringList )
     {
-      attr = QgsJsonUtils::parseArray( attr.toString(), QVariant::String );
+      attr = QgsJsonUtils::parseArray( attr.toString(), QMetaType::Type::QString );
     }
-    else if ( remoteLayer->fields().at( remoteAttributeIndex ).type() == QVariant::List )
+    else if ( remoteLayer->fields().at( remoteAttributeIndex ).type() == QMetaType::Type::QVariantList )
     {
       attr = QgsJsonUtils::parseArray( attr.toString(), remoteLayer->fields().at( remoteAttributeIndex ).subType() );
     }
@@ -1360,7 +1360,7 @@ QList<QgsField> QgsOfflineEditing::sqlQueryAttributesAdded( sqlite3 *db, const Q
   while ( ret == SQLITE_ROW )
   {
     const QgsField field( QString( reinterpret_cast< const char * >( sqlite3_column_text( stmt, 0 ) ) ),
-                          static_cast< QVariant::Type >( sqlite3_column_int( stmt, 1 ) ),
+                          static_cast< QMetaType::Type >( sqlite3_column_int( stmt, 1 ) ),
                           QString(), // typeName
                           sqlite3_column_int( stmt, 2 ),
                           sqlite3_column_int( stmt, 3 ),
@@ -1568,7 +1568,7 @@ void QgsOfflineEditing::committedAttributeValuesChanges( const QString &qgisLaye
     const QgsAttributeMap attrMap = cit.value();
     for ( QgsAttributeMap::const_iterator it = attrMap.constBegin(); it != attrMap.constEnd(); ++it )
     {
-      QString value = it.value().type() == QVariant::StringList || it.value().type() == QVariant::List ? QgsJsonUtils::encodeValue( it.value() ) : it.value().toString();
+      QString value = it.value().userType() == QMetaType::Type::QStringList || it.value().userType() == QMetaType::Type::QVariantList ? QgsJsonUtils::encodeValue( it.value() ) : it.value().toString();
       value.replace( QLatin1String( "'" ), QLatin1String( "''" ) ); // escape quote
       const QString sql = QStringLiteral( "INSERT INTO 'log_feature_updates' VALUES ( %1, %2, %3, %4, '%5' )" )
                           .arg( layerId )
@@ -1682,9 +1682,9 @@ int QgsOfflineEditing::getLayerPkIdx( const QgsVectorLayer *layer ) const
   if ( pkAttrs.length() == 1 )
   {
     const QgsField pkField = layer->fields().at( pkAttrs[0] );
-    const QVariant::Type pkType = pkField.type();
+    const QMetaType::Type pkType = pkField.type();
 
-    if ( pkType == QVariant::String )
+    if ( pkType == QMetaType::Type::QString )
     {
       return pkAttrs[0];
     }
