@@ -52,13 +52,30 @@ QgsProcessingDxfLayerDetailsWidget::QgsProcessingDxfLayerDetailsWidget( const QV
   if ( mLayer->fields().exists( layer.layerOutputAttributeIndex() ) )
     mFieldsComboBox->setField( mLayer->fields().at( layer.layerOutputAttributeIndex() ).name() );
 
+  mOverriddenName->setText( layer.overriddenName() );
+
+  if ( mLayer->geometryType() == Qgis::GeometryType::Point )
+  {
+    // Data defined blocks are only available for point layers
+    mGroupBoxBlocks->setVisible( true );
+    mGroupBoxBlocks->setChecked( layer.buildDataDefinedBlocks() );
+    mSpinBoxBlocks->setValue( layer.dataDefinedBlocksMaximumNumberOfClasses() );
+  }
+  else
+  {
+    mGroupBoxBlocks->setVisible( false );
+  }
+
   connect( mFieldsComboBox, &QgsFieldComboBox::fieldChanged, this, &QgsPanelWidget::widgetChanged );
+  connect( mOverriddenName, &QLineEdit::textChanged, this, &QgsPanelWidget::widgetChanged );
+  connect( mGroupBoxBlocks, &QGroupBox::toggled, this, &QgsPanelWidget::widgetChanged );
+  connect( mSpinBoxBlocks, &QSpinBox::textChanged, this, &QgsPanelWidget::widgetChanged );
 }
 
 QVariant QgsProcessingDxfLayerDetailsWidget::value() const
 {
   const int index = mLayer->fields().lookupField( mFieldsComboBox->currentField() );
-  const QgsDxfExport::DxfLayer layer( mLayer, index );
+  const QgsDxfExport::DxfLayer layer( mLayer, index, mGroupBoxBlocks->isChecked(), mSpinBoxBlocks->value(), mOverriddenName->text().trimmed() );
   return QgsProcessingParameterDxfLayers::layerAsVariantMap( layer );
 }
 
@@ -95,7 +112,7 @@ QgsProcessingDxfLayersPanelWidget::QgsProcessingDxfLayersPanelWidget(
     seenVectorLayers.insert( layer.layer() );
   }
 
-  const QList<QgsVectorLayer *> options = QgsProcessingUtils::compatibleVectorLayers( project, QList< int >() );
+  const QList<QgsVectorLayer *> options = QgsProcessingUtils::compatibleVectorLayers( project, QList< int >() << static_cast<int>( Qgis::ProcessingSourceType::VectorAnyGeometry ) );
   for ( const QgsVectorLayer *layer : options )
   {
     if ( seenVectorLayers.contains( layer ) )
@@ -104,6 +121,9 @@ QgsProcessingDxfLayersPanelWidget::QgsProcessingDxfLayersPanelWidget(
     QVariantMap vm;
     vm["layer"] = layer->id();
     vm["attributeIndex"] = -1;
+    vm["overriddenLayerName"] = QString();
+    vm["buildDataDefinedBlocks"] = DEFAULT_DXF_DATA_DEFINED_BLOCKS;
+    vm["dataDefinedBlocksMaximumNumberOfClasses"] = -1;
 
     const QString title = layer->name();
     addOption( vm, title, false );
@@ -166,8 +186,19 @@ QString QgsProcessingDxfLayersPanelWidget::titleForLayer( const QgsDxfExport::Dx
 {
   QString title = layer.layer()->name();
 
+  // if both options are set, the split attribute takes precedence,
+  // so hide overridden message to give users a hint on the result.
   if ( layer.layerOutputAttributeIndex() != -1 )
+  {
     title += tr( " [split attribute: %1]" ).arg( layer.splitLayerAttribute() );
+  }
+  else
+  {
+    if ( !layer.overriddenName().isEmpty() )
+    {
+      title += tr( " [overridden name: %1]" ).arg( layer.overriddenName() );
+    }
+  }
 
   return title;
 }
@@ -201,7 +232,7 @@ QgsProcessingDxfLayersWidget::QgsProcessingDxfLayersWidget( QWidget *parent )
 void QgsProcessingDxfLayersWidget::setValue( const QVariant &value )
 {
   if ( value.isValid() )
-    mValue = value.type() == QVariant::List ? value.toList() : QVariantList() << value;
+    mValue = value.userType() == QMetaType::Type::QVariantList ? value.toList() : QVariantList() << value;
   else
     mValue.clear();
 
