@@ -27,6 +27,7 @@
 #include "qgsrectangle.h"
 #include "qgscoordinatereferencesystem.h"
 #include "qgsnetworkaccessmanager.h"
+#include "qgssetrequestinitiator_p.h"
 #include "qgsnetworkreplyparser.h"
 #include "qgsmessagelog.h"
 #include "qgsexception.h"
@@ -1229,16 +1230,16 @@ bool QgsWcsProvider::calculateExtent() const
 }
 
 
-int QgsWcsProvider::capabilities() const
+Qgis::RasterInterfaceCapabilities QgsWcsProvider::capabilities() const
 {
-  int capability = NoCapabilities;
-  capability |= QgsRasterDataProvider::Identify;
-  capability |= QgsRasterDataProvider::IdentifyValue;
-  capability |= QgsRasterDataProvider::Prefetch;
+  Qgis::RasterInterfaceCapabilities capability = Qgis::RasterInterfaceCapability::NoCapabilities;
+  capability |= Qgis::RasterInterfaceCapability::Identify;
+  capability |= Qgis::RasterInterfaceCapability::IdentifyValue;
+  capability |= Qgis::RasterInterfaceCapability::Prefetch;
 
   if ( mHasSize )
   {
-    capability |= QgsRasterDataProvider::Size;
+    capability |= Qgis::RasterInterfaceCapability::Size;
   }
 
   return capability;
@@ -1616,9 +1617,9 @@ QString  QgsWcsProvider::description() const
   return WCS_DESCRIPTION;
 }
 
-QgsRasterDataProvider::ProviderCapabilities QgsWcsProvider::providerCapabilities() const
+Qgis::RasterProviderCapabilities QgsWcsProvider::providerCapabilities() const
 {
-  return ProviderCapability::ReloadData;
+  return Qgis::RasterProviderCapability::ReloadData;
 }
 
 void QgsWcsProvider::reloadProviderData()
